#!/usr/bin/perl -w

use Test::More 'no_plan';

package Catch;

sub TIEHANDLE {
    my($class, $var) = @_;
    return bless { var => $var }, $class;
}

sub PRINT  {
    my($self) = shift;
    ${'main::'.$self->{var}} .= join '', @_;
}

sub OPEN  {}    # XXX Hackery in case the user redirects
sub CLOSE {}    # XXX STDERR/STDOUT.  This is not the behavior we want.

sub READ {}
sub READLINE {}
sub GETC {}
sub BINMODE {}

my $Original_File = 'root/etc/e-smith/events/bootstrap-console-save/S50user-modify-quota';

package main;

# pre-5.8.0's warns aren't caught by a tied STDERR.
$SIG{__WARN__} = sub { $main::_STDERR_ .= join '', @_; };
tie *STDOUT, 'Catch', '_STDOUT_' or die $!;
tie *STDERR, 'Catch', '_STDERR_' or die $!;

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 53 root/etc/e-smith/events/bootstrap-console-save/S50user-modify-quota

use esmith::TestUtils qw(simulate_perl_program);
my $exit = simulate_perl_program($Original_File, 'wibble-event', 
                                                 'user_that_doesnt_exist');
is( $exit, 255 * 256,   'non-existent user - exit code' );
like( $@, qr{^Account "user_that_doesnt_exist" is not a user account},
                        '                    failure message');


$exit = simulate_perl_program($Original_File, 'wibble-event' );
is( $exit, 255 * 256,   'forgotten user - exit code' );
like( $@, qr{^Username argument missing},
                        '                 failure message');


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 85 root/etc/e-smith/events/bootstrap-console-save/S50user-modify-quota

my $a_tmp = '50-e-smith-quota/accounts.conf';
$ENV{ESMITH_ACCOUNT_DB} = $a_tmp;

END { unlink $a_tmp };

open(TMP, ">$a_tmp") || die $!;
print TMP <<'OUT';
root=system|Gid|0|Uid|0
server-manager=url
server-manual=url
shared=system|Gid|500|Visible|internal
shutdown=system
slocate=system|Gid|21
user_quota=user|Gid|1000|Uid|1000|MaxBlocksSoftLim|10000|MaxBlocks|20000|MaxFilesSoftLim|1000|MaxFiles|2000
user_partial=user|Gid|1001|Uid|1001|MaxBlocksSoftLim|123456|MaxFiles|321
user_no_quota=user|Gid|1002|Uid|1002
OUT
close TMP;

# Quota properties in the order setqlim likes.
my @QProps = qw(MaxBlocksSoftLim MaxBlocks MaxFilesSoftLim MaxFiles);
my %curr_quota = (
  1000  => { MaxFilesSoftLim => 100 },
  1001  => { MaxBlocks => 30000, MaxBlocksSoftLim => 20000 },
  1002  => { MaxFiles => 2 },
);

# What we expect the new quotas to be
my %expect_quotas = (
    1000 => { MaxBlocksSoftLim => 10000,  MaxBlocks => 20000,
              MaxFilesSoftLim  => 1000,   MaxFiles  => 2000 },
    1001 => { MaxBlocksSoftLim => 123456, MaxBlocks => 30000,
              MaxFilesSoftLim  => 0,      MaxFiles  => 321 },
    1002 => { MaxBlocksSoftLim => 0,      MaxBlocks => 0,
              MaxFilesSoftLim  => 0,      MaxFiles  => 2 },
);
            

{
    # Since these users don't really exist, we have to fool getpwnam
    # into using the Uid from the dummy accounts DB.
    no warnings 'once';
    local *CORE::GLOBAL::getpwnam = sub {
        esmith::AccountsDB->open->get($_[0])->prop('Uid');
    };


    # Override the Quota query and set functions since
    #   1) these users probably don't exist
    #   2) if they did, we don't want to screw with their quotas
    #   3) we need to know how each Quota function is called
    use Quota;
    no warnings 'redefine';

    my %query;
    sub Quota::query {
        my($dev, $uid, $isgrp) = @_;
        $query{$uid} = { isgrp => $isgrp };
        return (0, 
                $curr_quota{$uid}{MaxBlocksSoftLim} || 0,
                $curr_quota{$uid}{MaxBlocks} || 0,
                0,
                0,
                $curr_quota{$uid}{MaxFilesSoftLim} || 0,
                $curr_quota{$uid}{MaxFiles} || 0,
                0
               );
    }

    my %set_quota = ();
    sub Quota::setqlim {
        my($dev, $uid) = (shift, shift);
        my(@limits)    = splice(@_, 0, 4);
        my($tlo)       = shift;

        my %limits = ();
        @limits{@QProps} = @limits;
        $set_quota{$uid}{limits} = \%limits;
        $set_quota{$uid}{tlo}    = $tlo;

        return 1;
    }

    $_STDOUT_ = '';
    $_STDERR_ = '';
    my $exit = simulate_perl_program($Original_File, 'bootstrap-console-save');
    is( $exit, 0,       'bootstrap-console-save - exit code' );
    is( $_STDOUT_, '' );
    is( $_STDERR_, '' );
    is( $@, '' );

    is_deeply( [sort keys %set_quota], [sort qw(1000 1001 1002)] );
    foreach my $uid (keys %set_quota)
    {
        is_deeply( $set_quota{$uid}{limits}, $expect_quotas{$uid},
                                                   "setqlim for $uid");
    }
}


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

